;==========================================================
; DIMBEAR.LSP Copyright 1993 by Looking Glass Microproducts
;==========================================================
; Add bearings to Aligned Associative Dimensions 
;=============================================================
(defun C:DIMBEAR (/ ERROR PUSHVARS POPVARS SYSVARS OLD-ERROR 
                  NOTRANS ITEM ONE_DIMBEAR MAPSET TCHANGE 
                  DIMBEAR FILTER SS1)
   ;==========================================================
   ; Error Handler
   (defun ERROR (S)
      (if (not
             (member
                S
                '("Function cancelled" "console break")
             )
          )
         (princ S)
      )
      (POPVARS)
      (command ".undo" "end" ".undo" 1)
      (princ)
   )
   ;==========================================================
   ; Set and Save System Variables
   ;==========================================================
   (defun PUSHVARS (VLIST)
      (foreach PAIR VLIST
         (setq
            SYSVARS (cons
                       (cons
                          (strcase (car PAIR))
                          (getvar
                             (car PAIR)
                          )
                       )
                       SYSVARS
                    )
         )
         (if (cdr PAIR) (setvar (car PAIR) (cdr PAIR)))
      )
   )
   ;==========================================================
   ; Restore System Variables
   ;==========================================================
   (defun POPVARS ()
      (foreach PAIR SYSVARS (setvar (car PAIR) (cdr PAIR)))
      (setq
         *error* OLD-ERROR
      )
      (setq SYSVARS nil)
   )
   ;==========================================================
   ; Disallow transparent invocation of routine.
   ;==========================================================
   (defun NOTRANS ()
      (cond
         ((zerop (logand (getvar "cmdactive") (+ 1 2 4 8))))
         ((alert
             "This command may not be invoked transparently."
          )
         )
      )
   )
   ;==========================================================
   ; Item from association list
   ;==========================================================
   (defun ITEM (A B) (cdr (assoc A B)))
   ;==========================================================
   ;  Text Change -- Replace one substring with another
   ;==========================================================
   (defun TCHANGE (OLDSTR NEWSTR INSTRING ALL / I NLEN OLEN 
                   TEMPSTR)
      (setq I 1 NLEN (strlen NEWSTR) OLEN (strlen OLDSTR))
      (while (=
                OLEN
                (strlen
                   (setq TMPSTR (substr INSTRING I OLEN))
                )
             )
         (if (= TMPSTR OLDSTR)
            (setq
               INSTRING (strcat
                           (substr INSTRING 1 (1- I))
                           NEWSTR
                           (substr INSTRING (+ I OLEN))
                        )
               I        (if ALL (+ I NLEN) (strlen INSTRING))
            )
            (setq I (1+ I))
         )
      )
      INSTRING
   )
   ;==========================================================
   ; Add bearing to one dimension
   ;==========================================================
   (defun ONE_DIMBEAR (ENAME / ENT ANG)
      (setq
         ENT (entget ENAME)
         ANG (TCHANGE
                "d"
                "%%d"
                (angtos
                   (angle
                      (trans (ITEM 13 ENT) ENAME 1)
                      (trans
                         (ITEM 14 ENT)
                         ENAME
                         1
                      )
                   )
                )
                nil
             )
      )
      (entmod
         (subst
            (cons 1 (strcat ANG "  <>"))
            (assoc 1 ENT)
            ENT
         )
      )
   )
   ;==========================================================
   ; Apply func to every entity in ss
   ;==========================================================
   (defun MAPSET (FUNC SS PRMPT / I L SUFFIX ENAME)
      (if SS
         (progn
            (setq
               FUNC   (eval FUNC)
               I      0
               L      (sslength SS)
               SUFFIX (strcat ":" (itoa L))
            )
            (repeat
               L
               (setq I (1+ I) ENAME (ssname SS (- L I)))
               (if PRMPT
                  (setvar
                     "modemacro" (strcat
                                    PRMPT
                                    " "
                                    (itoa I)
                                    SUFFIX
                                 )
                  )
               )
               (FUNC ENAME)
            )
            SS
         )
      )
   )
   ;==========================================================
   ; Main Routine
   ;==========================================================
   (defun DIMBEAR ()
      (if SS1
         (prompt
            (strcat "\n" (itoa (sslength SS1)) " found")
         )
         (setq SS1 (ssget FILTER))
      )
      (cond
         ((null SS1))
         (t
            (setvar "highlight" 0)
            (MAPSET
               'ONE_DIMBEAR
               SS1
               "Processing"
            )
         )
      )
   )
   ;==========================================================
   ; Body of c:dimbear  
   ;==========================================================
   (if (NOTRANS)
      (progn
         (setq OLD-ERROR *error* *error* ERROR)
         (setvar "cmdecho" 0)
         (setq
            FILTER '((0 . "DIMENSION") (70 . 1))
            SS1    (ssget "i" FILTER)
         )
         (command ".undo" "group")
         (PUSHVARS
            '(("highlight") ("modemacro") ("unitmode" . 1))
         )
         (DIMBEAR)
         (POPVARS)
         (command ".undo" "end")
      )
   )
   (princ)
)
(princ
   (strcat
      "  DIMBEAR.LSP v1.0 (Copyright 1993 by "
      "Looking Glass Microproducts) loaded."
   )
)
(princ)
;end DIMBEAR.LSP
